#include "../headers/etc/StaticData.h"
#include "../headers/etc/Parameter.h"
#include "../headers/etc/Typedef.h"

#include "../headers/Model/LanguageModel.h"

#include "../headers/DataStructure/Phrase.h"

using namespace std;

// t[YꃂfRXgvZ
float LanguageModel::getLMScore(const Phrase& phrase) {
  return getLMScore(phrase.getLMWordIDArray());
}

float LanguageModel::getLMScoreForSentenceEnd(){
  copy_backward(idArray, idArray+historySize, idArray+historySize+1);
  *(idArray+historySize+1) = Vocab_None;
  *idArray = seTagID;

  return getNGramScore(idArray);
}

float LanguageModel::getLMScore(LMWordIDArray& sentence, const unsigned sentenceSize, const float threshold) const {
  // idx     : size-3   size-2   size-1     size
  // sentence: ...,     ssTagID, Vocab_None [eoa]

  unsigned start = sentenceSize - 3;  
  unsigned end   = sentenceSize - 1;
  
  // RXgvZ
  float score = 0.0F;

  do{
    score += (sentence[start] != -1) ? getNGramScore( &sentence[start] ) : unkWordPenalty;
    
    const unsigned* pos = &sentence[start];
    if( score < threshold ) return score;

    if( (end-start) >= nGramOrder ) sentence[--end] = Vocab_None;
  }while( 0 != start-- );

  return score;
}

// ꃂfRXgvZ(p)
float LanguageModel::getLMScore(const LMWordIDArray& newPhrase, const LMWordIDArray& history, bool debug){
  copy(newPhrase.begin(), newPhrase.end(), idArray);
  copy(history.begin(), history.end(), idArray+newPhrase.size());

  unsigned start = newPhrase.size() - 1;
  unsigned end   = newPhrase.size() + history.size();
  idArray[end]     = Vocab_None;

  if(debug){
    cerr << "new phrase: " ;
    for(unsigned i=0; i<newPhrase.size(); i++)
      cerr << getWord(newPhrase[i]) << "_";
    cerr << endl;
    
    cerr << "history: ";
    for(unsigned i=0; i<history.size(); i++)
      cerr << getWord(history[i]) << "_";
    cerr << endl;
    
    cerr << "idArray: ";
    for(unsigned i=0; i<end; i++)
      cerr << getWord(idArray[i]) << "_";
    cerr << endl;
  }

  // RXgvZ
  float score = 0.0F;
  do{
    score += (idArray[start] != -1) ? getNGramScore( &idArray[start] ) : unkWordPenalty;

    if(debug){
      cerr << "LM for: ";
      for(unsigned i=start; i<end; i++)
	cerr << getWord(idArray[i]) << "_";
      cerr << endl;
    }

    if( (end-start) >= nGramOrder ) idArray[--end] = Vocab_None;
  }while( 0 != start-- );

  historySize = end;

  return score;
}

// o萔
const float   LanguageModel::log10e      = (float)log10(exp(1.0));
const float   LanguageModel::unkWordPenalty= -100.0;
LMWordID LanguageModel::seTagID = 0;
LMWordID LanguageModel::ssTagID = 0;

// unsigned Coverage::size = 0;
