#ifndef PHRASEPAIR_H_
#define PHRASEPAIR_H_

#include "./Phrase.h"
#include "../Model/ReorderingModel.h"

using namespace std;

class PhrasePair{
 private:
  Phrase    srcPhrase;
  Phrase    tgtPhrase;
  float     score;
  ScoreArray tmScoreArray;
  ScoreArray rmScoreArray;
  float     futureScore;

 public:
  explicit PhrasePair(const string& src, const string& tgt, const float c, const float fc, const ScoreArray& tm = ScoreArray(0.0, 5), const ScoreArray& rm = ScoreArray(-0.1, StaticData::getInstance().getParameter().getRMFeatureNum()))
    : srcPhrase(src, SRC), tgtPhrase(tgt, TGT), score(c), futureScore(fc), tmScoreArray(tm), rmScoreArray(rm){}

  string toString() const {
    stringstream ss;
    string delimiter = StaticData::getInstance().getParameter().getDelimiter();
    ss << srcPhrase << delimiter << " ";
    ss << tgtPhrase << delimiter << " ";

    ss << "tm scores: ";
    for(unsigned i=0; i<tmScoreArray.size(); i++)
      ss << tmScoreArray[i] << " ";
    ss << delimiter << " ";
/*
    ss << "rm scores: ";
    for(unsigned i=0; i<rmScoreArray.size(); i++)
      ss << rmScoreArray[i] << " ";
    ss << delimiter << " ";
*/
    ss << "weighted score: " << score << " " << delimiter << " ";
    ss << "future score: " << futureScore << " ";
    return ss.str();
  }

  // operator
  bool operator<(const PhrasePair& rhs) const {
    return this->getFutureScore() < rhs.getFutureScore();
  }

  bool operator>(const PhrasePair& rhs) const {
    return this->getScore() > rhs.getScore();
  }

  // getter
  const Phrase& getSrcPhrase()  const { return srcPhrase; }
  const Phrase& getTgtPhrase()  const { return tgtPhrase; }
  float         getScore()        const { return score; }
  float         getFutureScore() const { return futureScore; }
  const ScoreArray& getRMScoreArray()   const{ return rmScoreArray; }

};

static ostream& operator<<(ostream& os, const PhrasePair& rhs) {
  return os << rhs.toString();
}

#endif /* PHRASEPAIR_H_ */
