#ifndef _REORDERING_MODEL_
#define _REORDERING_MODEL_

#include <iostream>
#include <deque>
#include <string>
#include <cmath>

#include "../etc/Utility.h"
#include "../DataStructure/Coverage.h"

using namespace std;

enum MsdBidirectionalFE{
  DIST_PENALTY = 0,
  MONO_BACKWORD,
  SWAP_BACKWORD,
  DISCONT_BACKWORD,
  MONO_FORWORD,
  SWAP_FORWARD,
  DISCONT_FORWARD,
  MSDBIFE_FEATURE_NUM
};

enum Type           {monotonic, msd, none};
enum Direction      {uni, bi};
enum Identification {f, fe};

class ReorderingType{
 public:
  Type           type;
  Direction      dir;
  Identification id;

 public:
  ReorderingType(): type(none), dir(uni) {}


  void set(const string& reorderingType){
    StringArray elems = split(reorderingType, "-");

    for(unsigned i=0; i<elems.size(); i++){
      string elem = elems[i];
      if (elem == "msd")                type = msd;
      else if (elem == "monotonicity")  type = monotonic;
      else if (elem == "bidirectional") dir  = bi;
      else if (elem == "f")             id   = f;
      else if (elem == "fe")            id   = fe;
    } // end of for
  }

  string toString() const {
    string strType;
    // reordering type
    switch(type){
    case monotonic:
      strType += "monotonicity-";  break;
    case msd:
      strType += "msd-";        break;
    } // end of switch

    // direction
    switch(dir){
    case uni:
      break;
    case bi:
      strType += "bidirectional-";  break;
    } // end of switch

    // identification
    switch(id){
    case f:
      strType += "f";    break;
    case fe:
      strType += "fe";   break;
    } // end of switch
    return strType;
  }

  void print() const {
    // reordering type
    switch(type){
    case monotonic:
      cerr << "monotonicity-";  break;
    case msd:
      cerr << "msd-";           break;
    } // end of switch

    // direction
    switch(dir){
    case uni:
      break;
    case bi:
      cerr << "bidirectional-";  break;
    } // end of switch

    // identification
    switch(id){
    case f:
      cerr << "f";    break;
    case fe:
      cerr << "fe";   break;
    } // end of switch

    cerr << "& distance" << endl;
  }
};



class ReorderingModel{
public:
  static const float calcFutureLinearDistPenalty2(const Coverage& coverage, const unsigned cardinality, int lastCoveredEnd) {
    Range r = {0, -1};
    float futureDistPenalty = 0.0F;
    
    while( !coverage.getNextUncoveredRange(r, r.end) ){
      futureDistPenalty += (float)abs(lastCoveredEnd - (int)r.start + 1);
      lastCoveredEnd  = (int)r.end;
      r.end++;
    }
    futureDistPenalty += abs(lastCoveredEnd - (int)Coverage::getSize() + 1);

    return futureDistPenalty;
  }

  static const float calcFutureLinearDistPenalty(const Coverage& coverage, const unsigned cardinality, int lastCoveredEnd) {
    Range r = {0,-1};
    if( !coverage.getNextUncoveredRange(r, r.end) ) return 0.0F;

    const unsigned firstUncoveredPosition = r.start;
    const int distortionOfNextJump = abs((int)lastCoveredEnd - (int)firstUncoveredPosition + 1);
    const int estimatedJumpTimes   = (int)cardinality - (int)firstUncoveredPosition + 1;
    
    return (float)(estimatedJumpTimes + distortionOfNextJump);
  }

  static const float calcLinearDistPenalty(const int lastPhraseEnd, const int newPhraseStart) {
    return (float)abs(lastPhraseEnd - newPhraseStart + 1);
  }
};

#endif
